import numpy as np
from abc import ABC, abstractmethod


class Error(ABC):
    def __init__(self):
        self.name = self.__class__.__name__

    def __call__(self, *args) -> float:
        return self._calcError(*args)

    @abstractmethod
    def _calcError(self, *args) -> float:
        pass


class Topological(Error):
    def __init__(self):
        super().__init__()

    def _calcError(self, bestMatchingIndices: np.ndarray, secondMatchingIndices: np.ndarray) -> float:
        distance = np.sum(np.abs(bestMatchingIndices - secondMatchingIndices), axis=-1) - 1
        return np.mean(distance)


class Quantazation(Error):
    def __init__(self):
        super().__init__()

    def _calcError(self, bestMatchingUnits: np.ndarray, dataBatch: np.ndarray) -> float:
        distance = np.linalg.norm(bestMatchingUnits - dataBatch, axis=-1)
        return np.mean(distance)